# encoding=utf-8


def key_to_hump(key):
	"""
		将变量转化为驼峰命名
	:return: 返回驼峰命名变量
	"""
	return key.lower().title().replace("_", "")


def check_class_method(obj, name):
	"""
		检查对象是否有某个属性
	:return:
	"""
	try:
		return getattr(obj, str(name))
	except Exception as e:
		print(f"对象{obj} 没有方法{str(name)}")
		return None



# encoding=utf-8
"""
Auth: coco369
Email: 779598160@qq.com

CreateTime: 2020/07/29

Desc: 配置文件
"""

# 爬虫配置
SPIDER_THREAD_COUNT = 1  # 爬虫并发数

# request网络请求超时时间
REQUEST_TIMEOUT = 22  # 等待服务器响应的超时时间，浮点数，或(connect timeout, read timeout)元组

# UserAgent类型 支持 'chrome', 'opera', 'firefox', 'internetexplorer', 'safari'，若不指定则随机类型
USER_AGENT_TYPE = "chrome"

# requests 使用session
USE_SESSION = False

# 下载时间间隔 单位秒。
# 支持格式, 如 SPIDER_SLEEP_TIME = 3
# 如 SPIDER_SLEEP_TIME = [2, 5] 或者 (2, 5) 则间隔为 2~5秒之间的随机数，包含2和5
SPIDER_SLEEP_TIME = (
    0
)



# encoding=utf-8
"""
Auth: coco369
Email: 779598160@qq.com

CreateTime: 2020/07/30

Desc: fastspider核心代码, 封装requests包
"""
import requests
import urllib3

from fastspider.http import user_agent
from fastspider.settings import common

# 忽略警告信息
urllib3.disable_warnings()


class Request(object):
	user_agent_pool = user_agent
	proxies_pool = None

	# 组装非必须传入的参数
	__request_attrs__ = ["headers", "data", "params", "cookies", "json", "timeout", "proxies", "verify"]
	# 组装默认传递的参数
	__default_requests_attrs__ = {
		"url": "",
		"retry_time": 0,
		"parser_name": None,
		"callback": None,
		"use_session": None,
		"download_middleware": None,
		"web_render": False,
		"web_render_time": 0,
		"request_sync": False
	}

	def __init__(self, url="", retry_time=0, parser_name=None, callback=None, use_session=None,
	             download_middleware=None, web_render=False, web_render_time=0, request_sync=False, **kwargs):
		self.url = url
		self.retry_time = retry_time
		self.parser_name = parser_name
		self.callback = callback
		self.request_sync = request_sync
		self.use_session = use_session
		self.download_middleware = download_middleware
		self.web_render = web_render
		self.web_render_time = web_render_time

		self.request_kwargs = {}
		for key, value in kwargs.items():
			if key in self.__class__.__request_attrs__:
				self.request_kwargs[key] = value

			self.__dict__[key] = value

	def get_response(self):

		# 设置请求超时时间
		self.request_kwargs.setdefault(
			"timeout", common.REQUEST_TIMEOUT
		)

		# 设置ssl验证
		self.request_kwargs.setdefault(
			"verify", False
		)

		# 获取请求方式method
		method = "GET"
		if not self.__dict__.get("method"):
			if "data" in self.request_kwargs:
				method = "POST"

		# 设置ua
		headers = self.request_kwargs.get("headers", {})
		if "user-agent" not in headers and "User-Agent" not in headers:
			headers["User-Agent"] = self.__class__.user_agent_pool.get_ua(common.USER_AGENT_TYPE)
			self.request_kwargs.update(headers=headers)

		# 设置session
		use_session = self.use_session if self.use_session else common.USE_SESSION

		if use_session:
			pass
		else:
			response = requests.request(method=method, url=self.url, **self.request_kwargs)
			# TODO: 封装response

		return response
